
/*

  Regenerage a tape file (including variable block counts) from a
  representation on the input stream created by `tape2out'.

  Representation is: each block of data to be preceded by an int, denoting
  the block size.

*/

#include <stdio.h>
#include <fcntl.h>

#define MAX_RECORD_SIZE 80000

static char *usage        = "USAGE: in2tape <device>\n" ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

void get_params (device, argc, argv)

char **device ;
int argc ;
char *argv [] ;

{
  if (argc != 2)
  {
    printf (usage) ;
    exit (1) ;
  }
  *device = argv [1] ;
}

void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char err_mess [128] ;

  char *device ;
  int src_desc, dst_desc ;
  int count = 0 ;

  get_params (&device, argc, argv) ;

  src_desc = 0 ;

  dst_desc = open (device, O_CREAT | O_TRUNC | O_WRONLY, 0x1a4) ;
  strcpy (err_mess, "Failed to open \"") ;
  strcat (err_mess, device) ;
  strcat (err_mess, "\"") ;
  sys_assert (dst_desc >= 0, err_mess) ;

  while (1)
  {
    char record [MAX_RECORD_SIZE] ;

    int result, len ;
    int ptr, residue ;

    result = read (src_desc, (char*) &len, sizeof (int)) ;
    sys_assert (result != 0, "Unexpected EOF") ;
    sys_assert (result == sizeof (int), "Read failed") ;

    if ((len < 0) || (MAX_RECORD_SIZE < len))
    {
      fprintf (stderr, "ERROR: block length = %d out of range\n") ;
      exit (1) ;
    }
    if (len == 0)
    {
      fprintf (stderr, "in2tape:%d records written\n", count) ;
      close (dst_desc) ;
      exit (0) ;
    }

    ptr     = 0 ;
    residue = len ;
    while (residue > 0)
    {
      result = read (src_desc, record + ptr, len) ;
      sys_assert (result >= 0, "Read failed") ;
      sys_assert (result != 0, "Unexpected EOF") ;
      ptr     += result ;
      residue -= result ;
    }

    result = write (dst_desc, record, len) ;
    sys_assert (result == len, "Write failed") ;
    count++ ;
  }
}
